#include "NIDAQmxBase.h"
#include <stdio.h>

void errinfo( char *func, int retval )
{
  char errBuff[2048]={'\0'};

  if ( retval != 0 ) {
    DAQmxBaseGetExtendedErrorInfo( errBuff, 2048 );
    printf( "Error in %s: %s\n", func, errBuff );
  }
}

double sample( char *chan ) {
  // Task parameters
  TaskHandle  taskHandle = 0;
  
  // Channel parameters
  float64     min = -10;
  float64     max = 10;
  
  // Timing parameters
  uInt64      samplesPerChan = 1;
  
  // Data read parameters
  double      data;
  int32       pointsToRead = 1;
  int32       pointsRead;
  float64     timeout = 10.0;

  errinfo( "task", DAQmxBaseCreateTask( "", &taskHandle ) );
  errinfo( "voltage",
	   DAQmxBaseCreateAIVoltageChan( taskHandle, chan, "",
					 DAQmx_Val_Cfg_Default,
					 min, max, DAQmx_Val_Volts,
					 NULL ) );
  errinfo( "start", DAQmxBaseStartTask( taskHandle ) );

  errinfo( "read",
	   DAQmxBaseReadAnalogF64( taskHandle, pointsToRead,
				   timeout, DAQmx_Val_GroupByChannel,
				   &data, samplesPerChan, &pointsRead,
				   NULL ) );

  errinfo( "stop", DAQmxBaseStopTask( taskHandle ) );
  errinfo( "clear", DAQmxBaseClearTask( taskHandle ) );

  if ( pointsRead != pointsToRead ) {
    fprintf( stderr, "Error: asked for %ld samples, got %ld.\n",
	     pointsToRead, pointsRead );
  }

  return data;
}

int samples = 10000;

int samplesecond_size( void ) {
  return samples;
}

void samplesecond( char *chan, double *data ) {
  // Task parameters
  TaskHandle  taskHandle = 0;
  
  // Channel parameters
  float64     min = -10;
  float64     max = 10;
  
  // Timing parameters
  float64     sampleRate     = 10000.0;
  
  // Data read parameters
  int32       pointsToRead = samples;
  int32       pointsRead;
  float64     timeout = 10.0;

  errinfo( "task", DAQmxBaseCreateTask( "", &taskHandle ) );
  errinfo( "voltage",
	   DAQmxBaseCreateAIVoltageChan( taskHandle, chan, "",
					 DAQmx_Val_Cfg_Default,
					 min, max, DAQmx_Val_Volts,
					 NULL ) );
  errinfo( "clock",
	   DAQmxBaseCfgSampClkTiming( taskHandle, "OnboardClock", sampleRate,
				      DAQmx_Val_Rising, DAQmx_Val_FiniteSamps,
				      samples ) );

  errinfo( "start", DAQmxBaseStartTask( taskHandle ) );

  errinfo( "read",
	   DAQmxBaseReadAnalogF64( taskHandle, pointsToRead,
				   timeout, DAQmx_Val_GroupByChannel,
				   data, samples, &pointsRead,
				   NULL ) );

  errinfo( "stop", DAQmxBaseStopTask( taskHandle ) );
  errinfo( "clear", DAQmxBaseClearTask( taskHandle ) );

  if ( pointsRead != pointsToRead ) {
    fprintf( stderr, "Error: asked for %ld samples, got %ld.\n",
	     pointsToRead, pointsRead );
  }
}
